﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Text;
using SampleShop;
using SampleShop.Products;

namespace Hardcodet.Wpf.Samples.ViewModel
{
  /// <summary>
  /// Provides a simplified view model for the sample
  /// application.
  /// </summary>
  public class ShopModel : INotifyPropertyChanged
  {
    private ObservableCollection<ShopCategory> categories;

    public ObservableCollection<ShopCategory> Categories
    {
      get { return categories; }
      set {
        if (categories != null)
        {
          categories.CollectionChanged -= OnRootCollectionChanged;
        }

        categories = value;
        if (value != null)
        {
          value.CollectionChanged += OnRootCollectionChanged;
        }

        RaisePropertyChangedEvent("Categories");
      }
    }


    private void OnRootCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
    {
      RaisePropertyChangedEvent("Categories");
    }



    /// <summary>
    /// Refreshes the data.
    /// </summary>
    public void RefreshData()
    {
      Categories = ShopUtil.CreateShop();
    }


    /// <summary>
    /// Creates a bigger tree that consists of numerical categories.
    /// </summary>
    public void CreateDummyData()
    {
      ObservableCollection<ShopCategory> list = new ObservableCollection<ShopCategory>();
      for (int i = 1; i <= 10; i++)
      {
        ShopCategory cat = new ShopCategory(i.ToString(), null);
        list.Add(cat);
        for (int j = 0; j < 1000; j++)
        {
          int index = 1000 * i + j;
          ShopCategory sub = new ShopCategory(index.ToString(), cat);
          cat.SubCategories.Add(sub);
        }
      }

      Categories = list;
    }


    ///<summary>
    ///Occurs when a property value changes.
    ///</summary>
    public event PropertyChangedEventHandler PropertyChanged;


    /// <summary>
    /// Fires the <see cref="PropertyChanged"/> event for a
    /// given property.
    /// </summary>
    /// <param name="propertyName">The changed property.</param>
    private void RaisePropertyChangedEvent(string propertyName)
    {
      if (PropertyChanged != null)
      {
        PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
      }
    }


    public ShopModel()
    {
      RefreshData();
    }

    public ShopCategory TryFindCategoryByName(string categoryName)
    {
      return TryFindCategoryByName(null, categoryName);
    }


    public ShopCategory TryFindCategoryByName(ShopCategory parent, string categoryName)
    {
      ObservableCollection<ShopCategory> cats;
      cats = parent == null ? categories : parent.SubCategories;
      foreach (ShopCategory category in cats)
      {
        if (category.CategoryName == categoryName)
        {
          return category;
        }
        else
        {
          ShopCategory cat = TryFindCategoryByName(category, categoryName);
          if (cat != null) return cat;
        }
      }

      return null;
    }
  }
}
